# -*- coding: utf-8 -*-

__revision__ = '$Id: scriptnames.py 7386 2006-04-30 11:12:55Z matt $'

scriptNames = {
	u'Yiii': u'Yi',
	u'Telu': u'Telugu',
	u'Taml': u'Tamil',
	u'Guru': u'Gurmukhi',
	u'Hebr': u'Hebrew',
	u'Geor': u'Georgian (Mkhedruli)',
	u'Ugar': u'Ugaritic',
	u'Cyrl': u'Cyrillic',
	u'Hrkt': u'Kanji & Kana',
	u'Armn': u'Armenian',
	u'Runr': u'Runic',
	u'Khmr': u'Khmer',
	u'Latn': u'Latin',
	u'Hani': u'Han (Hanzi, Kanji, Hanja)',
	u'Ital': u'Old Italic (Etruscan, Oscan, etc.)',
	u'Hano': u'Hanunoo (Hanunóo)',
	u'Ethi': u'Ethiopic (Ge\'ez)',
	u'Gujr': u'Gujarati',
	u'Hang': u'Hangul',
	u'Arab': u'Arabic',
	u'Thaa': u'Thaana',
	u'Buhd': u'Buhid',
	u'Sinh': u'Sinhala',
	u'Orya': u'Oriya',
	u'Hans': u'Han (Simplified variant)',
	u'Thai': u'Thai',
	u'Cprt': u'Cypriot',
	u'Linb': u'Linear B',
	u'Hant': u'Han (Traditional variant)',
	u'Osma': u'Osmanya',
	u'Mong': u'Mongolian',
	u'Deva': u'Devanagari (Nagari)',
	u'Laoo': u'Lao',
	u'Tagb': u'Tagbanwa',
	u'Hira': u'Hiragana',
	u'Bopo': u'Bopomofo',
	u'Goth': u'Gothic',
	u'Tale': u'Tai Le',
	u'Mymr': u'Myanmar (Burmese)',
	u'Tglg': u'Tagalog',
	u'Grek': u'Greek',
	u'Mlym': u'Malayalam',
	u'Cher': u'Cherokee',
	u'Tibt': u'Tibetan',
	u'Kana': u'Katakana',
	u'Syrc': u'Syriac',
	u'Cans': u'Unified Canadian Aboriginal Syllabics',
	u'Beng': u'Bengali',
	u'Limb': u'Limbu',
	u'Ogam': u'Ogham',
	u'Knda': u'Kannada',
}

# EOF
