# -*- coding: utf-8 -*-

__revision__ = '$Id: languagenames.py 8725 2006-12-17 22:39:07Z luks $'

languageNames = {
	u'ART': u'Artificial (Other)',
	u'ROH': u'Raeto-Romance',
	u'SCO': u'Scots',
	u'SCN': u'Sicilian',
	u'ROM': u'Romany',
	u'RON': u'Romanian',
	u'OSS': u'Ossetian; Ossetic',
	u'ALE': u'Aleut',
	u'MNI': u'Manipuri',
	u'NWC': u'Classical Newari; Old Newari; Classical Nepal Bhasa',
	u'OSA': u'Osage',
	u'MNC': u'Manchu',
	u'MWR': u'Marwari',
	u'VEN': u'Venda',
	u'MWL': u'Mirandese',
	u'FAS': u'Persian',
	u'FAT': u'Fanti',
	u'FAN': u'Fang',
	u'FAO': u'Faroese',
	u'DIN': u'Dinka',
	u'HYE': u'Armenian',
	u'DSB': u'Lower Sorbian',
	u'CAR': u'Carib',
	u'DIV': u'Divehi',
	u'TEL': u'Telugu',
	u'TEM': u'Timne',
	u'NBL': u'Ndebele, South; South Ndebele',
	u'TER': u'Tereno',
	u'TET': u'Tetum',
	u'SUN': u'Sundanese',
	u'KUT': u'Kutenai',
	u'SUK': u'Sukuma',
	u'KUR': u'Kurdish',
	u'KUM': u'Kumyk',
	u'SUS': u'Susu',
	u'NEW': u'Newari; Nepal Bhasa',
	u'KUA': u'Kuanyama; Kwanyama',
	u'MEN': u'Mende',
	u'LEZ': u'Lezghian',
	u'GLA': u'Gaelic; Scottish Gaelic',
	u'BOS': u'Bosnian',
	u'GLE': u'Irish',
	u'EKA': u'Ekajuk',
	u'GLG': u'Gallegan',
	u'AKA': u'Akan',
	u'BOD': u'Tibetan',
	u'GLV': u'Manx',
	u'JRB': u'Judeo-Arabic',
	u'VIE': u'Vietnamese',
	u'IPK': u'Inupiaq',
	u'UZB': u'Uzbek',
	u'BRE': u'Breton',
	u'BRA': u'Braj',
	u'AYM': u'Aymara',
	u'CHA': u'Chamorro',
	u'CHB': u'Chibcha',
	u'CHE': u'Chechen',
	u'CHG': u'Chagatai',
	u'CHK': u'Chuukese',
	u'CHM': u'Mari',
	u'CHN': u'Chinook jargon',
	u'CHO': u'Choctaw',
	u'CHP': u'Chipewyan',
	u'CHR': u'Cherokee',
	u'CHU': u'Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic',
	u'CHV': u'Chuvash',
	u'CHY': u'Cheyenne',
	u'MSA': u'Malay',
	u'III': u'Sichuan Yi',
	u'ACE': u'Achinese',
	u'IBO': u'Igbo',
	u'IBA': u'Iban',
	u'XHO': u'Xhosa',
	u'DEU': u'German',
	u'CAT': u'Catalan; Valencian',
	u'DEL': u'Delaware',
	u'DEN': u'Slave (Athapascan)',
	u'CAD': u'Caddo',
	u'TAT': u'Tatar',
	u'RAJ': u'Rajasthani',
	u'SPA': u'Spanish; Castilian',
	u'TAM': u'Tamil',
	u'TAH': u'Tahitian',
	u'AFH': u'Afrihili',
	u'ENG': u'English',
	u'CSB': u'Kashubian',
	u'NYN': u'Nyankole',
	u'NYO': u'Nyoro',
	u'SID': u'Sidamo',
	u'NYA': u'Chichewa; Chewa; Nyanja',
	u'SIN': u'Sinhala; Sinhalese',
	u'AFR': u'Afrikaans',
	u'LAM': u'Lamba',
	u'SND': u'Sindhi',
	u'MAR': u'Marathi',
	u'LAH': u'Lahnda',
	u'NYM': u'Nyamwezi',
	u'SNA': u'Shona',
	u'LAD': u'Ladino',
	u'SNK': u'Soninke',
	u'MAD': u'Madurese',
	u'MAG': u'Magahi',
	u'MAI': u'Maithili',
	u'MAH': u'Marshallese',
	u'LAV': u'Latvian',
	u'MAL': u'Malayalam',
	u'MAN': u'Mandingo',
	u'ZND': u'Zande',
	u'ZEN': u'Zenaga',
	u'KBD': u'Kabardian',
	u'ITA': u'Italian',
	u'VAI': u'Vai',
	u'TSN': u'Tswana',
	u'TSO': u'Tsonga',
	u'TSI': u'Tsimshian',
	u'BYN': u'Blin; Bilin',
	u'FIJ': u'Fijian',
	u'FIN': u'Finnish',
	u'EUS': u'Basque',
	u'CEB': u'Cebuano',
	u'DAN': u'Danish',
	u'NOG': u'Nogai',
	u'NOB': u'Norwegian Bokmål; Bokmål, Norwegian',
	u'DAK': u'Dakota',
	u'CES': u'Czech',
	u'DAR': u'Dargwa',
	u'DAY': u'Dayak',
	u'NOR': u'Norwegian',
	u'KPE': u'Kpelle',
	u'GUJ': u'Gujarati',
	u'MDF': u'Moksha',
	u'MAS': u'Masai',
	u'LAO': u'Lao',
	u'MDR': u'Mandar',
	u'GON': u'Gondi',
	u'SMS': u'Skolt Sami',
	u'SMO': u'Samoan',
	u'SMN': u'Inari Sami',
	u'SMJ': u'Lule Sami',
	u'GOT': u'Gothic',
	u'SME': u'Northern Sami',
	u'BLA': u'Siksika',
	u'SMA': u'Southern Sami',
	u'GOR': u'Gorontalo',
	u'AST': u'Asturian; Bable',
	u'ORM': u'Oromo',
	u'QUE': u'Quechua',
	u'ORI': u'Oriya',
	u'CRH': u'Crimean Tatar; Crimean Turkish',
	u'ASM': u'Assamese',
	u'PUS': u'Pushto',
	u'DGR': u'Dogrib',
	u'LTZ': u'Luxembourgish; Letzeburgesch',
	u'NDO': u'Ndonga',
	u'GEZ': u'Geez',
	u'ISL': u'Icelandic',
	u'LAT': u'Latin',
	u'MAK': u'Makasar',
	u'ZAP': u'Zapotec',
	u'YID': u'Yiddish',
	u'KOK': u'Konkani',
	u'KOM': u'Komi',
	u'KON': u'Kongo',
	u'UKR': u'Ukrainian',
	u'TON': u'Tonga (Tonga Islands)',
	u'KOS': u'Kosraean',
	u'KOR': u'Korean',
	u'TOG': u'Tonga (Nyasa)',
	u'HUN': u'Hungarian',
	u'HUP': u'Hupa',
	u'CYM': u'Welsh',
	u'UDM': u'Udmurt',
	u'BEJ': u'Beja',
	u'BEN': u'Bengali',
	u'BEL': u'Belarusian',
	u'BEM': u'Bemba',
	u'AAR': u'Afar',
	u'NZI': u'Nzima',
	u'SAH': u'Yakut',
	u'SAN': u'Sanskrit',
	u'SAM': u'Samaritan Aramaic',
	u'SAG': u'Sango',
	u'SAD': u'Sandawe',
	u'RAR': u'Rarotongan',
	u'RAP': u'Rapanui',
	u'SAS': u'Sasak',
	u'SAT': u'Santali',
	u'MIN': u'Minangkabau',
	u'LIM': u'Limburgan; Limburger; Limburgish',
	u'LIN': u'Lingala',
	u'LIT': u'Lithuanian',
	u'EFI': u'Efik',
	u'BTK': u'Batak (Indonesia)',
	u'KAC': u'Kachin',
	u'KAB': u'Kabyle',
	u'KAA': u'Kara-Kalpak',
	u'KAN': u'Kannada',
	u'KAM': u'Kamba',
	u'KAL': u'Kalaallisut; Greenlandic',
	u'KAS': u'Kashmiri',
	u'KAR': u'Karen',
	u'KAU': u'Kanuri',
	u'KAT': u'Georgian',
	u'KAZ': u'Kazakh',
	u'TYV': u'Tuvinian',
	u'AWA': u'Awadhi',
	u'URD': u'Urdu',
	u'DOI': u'Dogri',
	u'TPI': u'Tok Pisin',
	u'MRI': u'Maori',
	u'ABK': u'Abkhazian',
	u'TKL': u'Tokelau',
	u'NLD': u'Dutch; Flemish',
	u'OJI': u'Ojibwa',
	u'OCI': u'Occitan (post 1500); Provençal',
	u'WOL': u'Wolof',
	u'JAV': u'Javanese',
	u'HRV': u'Croatian',
	u'DYU': u'Dyula',
	u'SSW': u'Swati',
	u'MUL': u'Multiple languages',
	u'HIL': u'Hiligaynon',
	u'HIM': u'Himachali',
	u'HIN': u'Hindi',
	u'BAS': u'Basa',
	u'GBA': u'Gbaya',
	u'WLN': u'Walloon',
	u'BAD': u'Banda',
	u'NEP': u'Nepali',
	u'CRE': u'Cree',
	u'BAN': u'Balinese',
	u'BAL': u'Baluchi',
	u'BAM': u'Bambara',
	u'BAK': u'Bashkir',
	u'SHN': u'Shan',
	u'ARP': u'Arapaho',
	u'ARW': u'Arawak',
	u'ARA': u'Arabic',
	u'ARC': u'Aramaic',
	u'ARG': u'Aragonese',
	u'SEL': u'Selkup',
	u'ARN': u'Araucanian',
	u'LUS': u'Lushai',
	u'MUS': u'Creek',
	u'LUA': u'Luba-Lulua',
	u'LUB': u'Luba-Katanga',
	u'LUG': u'Ganda',
	u'LUI': u'Luiseno',
	u'LUN': u'Lunda',
	u'LUO': u'Luo (Kenya and Tanzania)',
	u'IKU': u'Inuktitut',
	u'TUR': u'Turkish',
	u'TUK': u'Turkmen',
	u'TUM': u'Tumbuka',
	u'COP': u'Coptic',
	u'COS': u'Corsican',
	u'COR': u'Cornish',
	u'ILO': u'Iloko',
	u'GWI': u'Gwich´in',
	u'TLI': u'Tlingit',
	u'TLH': u'Klingon; tlhIngan-Hol',
	u'POR': u'Portuguese',
	u'PON': u'Pohnpeian',
	u'POL': u'Polish',
	u'TGK': u'Tajik',
	u'TGL': u'Tagalog',
	u'FRA': u'French',
	u'BHO': u'Bhojpuri',
	u'SWA': u'Swahili',
	u'DUA': u'Duala',
	u'SWE': u'Swedish',
	u'YAP': u'Yapese',
	u'TIV': u'Tiv',
	u'YAO': u'Yao',
	u'XAL': u'Kalmyk',
	u'FRY': u'Frisian',
	u'GAY': u'Gayo',
	u'OTA': u'Turkish, Ottoman (1500-1928)',
	u'HMN': u'Hmong',
	u'HMO': u'Hiri Motu',
	u'GAA': u'Ga',
	u'FUR': u'Friulian',
	u'MLG': u'Malagasy',
	u'SLV': u'Slovenian',
	u'FIL': u'Filipino; Pilipino',
	u'MLT': u'Maltese',
	u'SLK': u'Slovak',
	u'FUL': u'Fulah',
	u'JPN': u'Japanese',
	u'VOL': u'Volapük',
	u'VOT': u'Votic',
	u'IND': u'Indonesian',
	u'AVE': u'Avestan',
	u'JPR': u'Judeo-Persian',
	u'AVA': u'Avaric',
	u'PAP': u'Papiamento',
	u'EWO': u'Ewondo',
	u'PAU': u'Palauan',
	u'EWE': u'Ewe',
	u'PAG': u'Pangasinan',
	u'PAM': u'Pampanga',
	u'PAN': u'Panjabi; Punjabi',
	u'KIR': u'Kirghiz',
	u'NIA': u'Nias',
	u'KIK': u'Kikuyu; Gikuyu',
	u'SYR': u'Syriac',
	u'KIN': u'Kinyarwanda',
	u'NIU': u'Niuean',
	u'EPO': u'Esperanto',
	u'JBO': u'Lojban',
	u'MIC': u'Mi\'kmaq; Micmac',
	u'THA': u'Thai',
	u'HAI': u'Haida',
	u'ELL': u'Greek, Modern (1453-)',
	u'ADY': u'Adyghe; Adygei',
	u'ELX': u'Elamite',
	u'ADA': u'Adangme',
	u'GRB': u'Grebo',
	u'HAT': u'Haitian; Haitian Creole',
	u'HAU': u'Hausa',
	u'HAW': u'Hawaiian',
	u'BIN': u'Bini',
	u'AMH': u'Amharic',
	u'BIK': u'Bikol',
	u'BIH': u'Bihari',
	u'MOS': u'Mossi',
	u'MOH': u'Mohawk',
	u'MON': u'Mongolian',
	u'MOL': u'Moldavian',
	u'BIS': u'Bislama',
	u'TVL': u'Tuvalu',
	u'IJO': u'Ijo',
	u'EST': u'Estonian',
	u'KMB': u'Kimbundu',
	u'UMB': u'Umbundu',
	u'TMH': u'Tamashek',
	u'FON': u'Fon',
	u'HSB': u'Upper Sorbian',
	u'RUN': u'Rundi',
	u'RUS': u'Russian',
	u'PLI': u'Pali',
	u'SRD': u'Sardinian',
	u'ACH': u'Acoli',
	u'NDE': u'Ndebele, North; North Ndebele',
	u'DZO': u'Dzongkha',
	u'KRU': u'Kurukh',
	u'SRR': u'Serer',
	u'IDO': u'Ido',
	u'SRP': u'Serbian',
	u'KRO': u'Kru',
	u'KRC': u'Karachay-Balkar',
	u'NDS': u'Low German; Low Saxon; German, Low; Saxon, Low',
	u'ZUN': u'Zuni',
	u'ZUL': u'Zulu',
	u'TWI': u'Twi',
	u'NSO': u'Northern Sotho, Pedi; Sepedi',
	u'SOM': u'Somali',
	u'SON': u'Songhai',
	u'SOT': u'Sotho, Southern',
	u'MKD': u'Macedonian',
	u'HER': u'Herero',
	u'LOL': u'Mongo',
	u'HEB': u'Hebrew',
	u'LOZ': u'Lozi',
	u'GIL': u'Gilbertese',
	u'WAS': u'Washo',
	u'WAR': u'Waray',
	u'BUL': u'Bulgarian',
	u'WAL': u'Walamo',
	u'BUA': u'Buriat',
	u'BUG': u'Buginese',
	u'AZE': u'Azerbaijani',
	u'ZHA': u'Zhuang; Chuang',
	u'ZHO': u'Chinese',
	u'NNO': u'Norwegian Nynorsk; Nynorsk, Norwegian',
	u'UIG': u'Uighur; Uyghur',
	u'MYV': u'Erzya',
	u'INH': u'Ingush',
	u'KHM': u'Khmer',
	u'MYA': u'Burmese',
	u'KHA': u'Khasi',
	u'INA': u'Interlingua (International Auxiliary Language Association)',
	u'NAH': u'Nahuatl',
	u'TIR': u'Tigrinya',
	u'NAP': u'Neapolitan',
	u'NAV': u'Navajo; Navaho',
	u'NAU': u'Nauru',
	u'GRN': u'Guarani',
	u'TIG': u'Tigre',
	u'YOR': u'Yoruba',
	u'ILE': u'Interlingue',
	u'SQI': u'Albanian',
}

# EOF
