# -*- coding: utf-8 -*-

__revision__ = '$Id: releasetypenames.py 8728 2006-12-17 23:42:30Z luks $'

releaseTypeNames = {
	u'http://musicbrainz.org/ns/mmd-1.0#None': u'None',
	u'http://musicbrainz.org/ns/mmd-1.0#Album': u'Album',
	u'http://musicbrainz.org/ns/mmd-1.0#Single': u'Single',
	u'http://musicbrainz.org/ns/mmd-1.0#EP': u'EP',
	u'http://musicbrainz.org/ns/mmd-1.0#Compilation': u'Compilation',
	u'http://musicbrainz.org/ns/mmd-1.0#Soundtrack': u'Soundtrack',
	u'http://musicbrainz.org/ns/mmd-1.0#Spokenword': u'Spokenword',
	u'http://musicbrainz.org/ns/mmd-1.0#Interview': u'Interview',
	u'http://musicbrainz.org/ns/mmd-1.0#Audiobook': u'Audiobook',
	u'http://musicbrainz.org/ns/mmd-1.0#Live': u'Live',
	u'http://musicbrainz.org/ns/mmd-1.0#Remix': u'Remix',
	u'http://musicbrainz.org/ns/mmd-1.0#Other': u'Other',
	u'http://musicbrainz.org/ns/mmd-1.0#Official': u'Official',
	u'http://musicbrainz.org/ns/mmd-1.0#Promotion': u'Promotion',
	u'http://musicbrainz.org/ns/mmd-1.0#Bootleg': u'Bootleg',
	u'http://musicbrainz.org/ns/mmd-1.0#Pseudo-Release': u'Pseudo-Release',
}

# EOF
