# -*- coding: utf-8 -*-

__revision__ = '$Id: countrynames.py 7386 2006-04-30 11:12:55Z matt $'

countryNames = {
	u'BD': u'Bangladesh',
	u'BE': u'Belgium',
	u'BF': u'Burkina Faso',
	u'BG': u'Bulgaria',
	u'BB': u'Barbados',
	u'WF': u'Wallis and Futuna Islands',
	u'BM': u'Bermuda',
	u'BN': u'Brunei Darussalam',
	u'BO': u'Bolivia',
	u'BH': u'Bahrain',
	u'BI': u'Burundi',
	u'BJ': u'Benin',
	u'BT': u'Bhutan',
	u'JM': u'Jamaica',
	u'BV': u'Bouvet Island',
	u'BW': u'Botswana',
	u'WS': u'Samoa',
	u'BR': u'Brazil',
	u'BS': u'Bahamas',
	u'BY': u'Belarus',
	u'BZ': u'Belize',
	u'RU': u'Russian Federation',
	u'RW': u'Rwanda',
	u'RE': u'Reunion',
	u'TM': u'Turkmenistan',
	u'TJ': u'Tajikistan',
	u'RO': u'Romania',
	u'TK': u'Tokelau',
	u'GW': u'Guinea-Bissau',
	u'GU': u'Guam',
	u'GT': u'Guatemala',
	u'GR': u'Greece',
	u'GQ': u'Equatorial Guinea',
	u'GP': u'Guadeloupe',
	u'JP': u'Japan',
	u'GY': u'Guyana',
	u'GF': u'French Guiana',
	u'GE': u'Georgia',
	u'GD': u'Grenada',
	u'GB': u'United Kingdom',
	u'GA': u'Gabon',
	u'SV': u'El Salvador',
	u'GN': u'Guinea',
	u'GM': u'Gambia',
	u'GL': u'Greenland',
	u'GI': u'Gibraltar',
	u'GH': u'Ghana',
	u'OM': u'Oman',
	u'TN': u'Tunisia',
	u'JO': u'Jordan',
	u'HT': u'Haiti',
	u'HU': u'Hungary',
	u'HK': u'Hong Kong',
	u'HN': u'Honduras',
	u'HM': u'Heard and Mc Donald Islands',
	u'VE': u'Venezuela',
	u'PR': u'Puerto Rico',
	u'PW': u'Palau',
	u'PT': u'Portugal',
	u'SJ': u'Svalbard and Jan Mayen Islands',
	u'PY': u'Paraguay',
	u'IQ': u'Iraq',
	u'PA': u'Panama',
	u'PF': u'French Polynesia',
	u'PG': u'Papua New Guinea',
	u'PE': u'Peru',
	u'PK': u'Pakistan',
	u'PH': u'Philippines',
	u'PN': u'Pitcairn',
	u'PL': u'Poland',
	u'PM': u'St. Pierre and Miquelon',
	u'ZM': u'Zambia',
	u'EH': u'Western Sahara',
	u'EE': u'Estonia',
	u'EG': u'Egypt',
	u'ZA': u'South Africa',
	u'EC': u'Ecuador',
	u'IT': u'Italy',
	u'VN': u'Viet Nam',
	u'SB': u'Solomon Islands',
	u'ET': u'Ethiopia',
	u'SO': u'Somalia',
	u'ZW': u'Zimbabwe',
	u'SA': u'Saudi Arabia',
	u'ES': u'Spain',
	u'ER': u'Eritrea',
	u'MD': u'Moldova, Republic of',
	u'MG': u'Madagascar',
	u'MA': u'Morocco',
	u'MC': u'Monaco',
	u'UZ': u'Uzbekistan',
	u'MM': u'Myanmar',
	u'ML': u'Mali',
	u'MO': u'Macau',
	u'MN': u'Mongolia',
	u'MH': u'Marshall Islands',
	u'MK': u'Macedonia, The Former Yugoslav Republic of',
	u'MU': u'Mauritius',
	u'MT': u'Malta',
	u'MW': u'Malawi',
	u'MV': u'Maldives',
	u'MQ': u'Martinique',
	u'MP': u'Northern Mariana Islands',
	u'MS': u'Montserrat',
	u'MR': u'Mauritania',
	u'UG': u'Uganda',
	u'MY': u'Malaysia',
	u'MX': u'Mexico',
	u'IL': u'Israel',
	u'FR': u'France',
	u'IO': u'British Indian Ocean Territory',
	u'SH': u'St. Helena',
	u'FI': u'Finland',
	u'FJ': u'Fiji',
	u'FK': u'Falkland Islands (Malvinas)',
	u'FM': u'Micronesia, Federated States of',
	u'FO': u'Faroe Islands',
	u'NI': u'Nicaragua',
	u'NL': u'Netherlands',
	u'NO': u'Norway',
	u'NA': u'Namibia',
	u'VU': u'Vanuatu',
	u'NC': u'New Caledonia',
	u'NE': u'Niger',
	u'NF': u'Norfolk Island',
	u'NG': u'Nigeria',
	u'NZ': u'New Zealand',
	u'ZR': u'Zaire',
	u'NP': u'Nepal',
	u'NR': u'Nauru',
	u'NU': u'Niue',
	u'CK': u'Cook Islands',
	u'CI': u'Cote d\'Ivoire',
	u'CH': u'Switzerland',
	u'CO': u'Colombia',
	u'CN': u'China',
	u'CM': u'Cameroon',
	u'CL': u'Chile',
	u'CC': u'Cocos (Keeling) Islands',
	u'CA': u'Canada',
	u'CG': u'Congo',
	u'CF': u'Central African Republic',
	u'CZ': u'Czech Republic',
	u'CY': u'Cyprus',
	u'CX': u'Christmas Island',
	u'CR': u'Costa Rica',
	u'CV': u'Cape Verde',
	u'CU': u'Cuba',
	u'SZ': u'Swaziland',
	u'SY': u'Syrian Arab Republic',
	u'KG': u'Kyrgyzstan',
	u'KE': u'Kenya',
	u'SR': u'Suriname',
	u'KI': u'Kiribati',
	u'KH': u'Cambodia',
	u'KN': u'Saint Kitts and Nevis',
	u'KM': u'Comoros',
	u'ST': u'Sao Tome and Principe',
	u'SI': u'Slovenia',
	u'KW': u'Kuwait',
	u'SN': u'Senegal',
	u'SM': u'San Marino',
	u'SL': u'Sierra Leone',
	u'SC': u'Seychelles',
	u'KZ': u'Kazakhstan',
	u'KY': u'Cayman Islands',
	u'SG': u'Singapore',
	u'SE': u'Sweden',
	u'SD': u'Sudan',
	u'DO': u'Dominican Republic',
	u'DM': u'Dominica',
	u'DJ': u'Djibouti',
	u'DK': u'Denmark',
	u'VG': u'Virgin Islands (British)',
	u'DE': u'Germany',
	u'YE': u'Yemen',
	u'DZ': u'Algeria',
	u'US': u'United States',
	u'UY': u'Uruguay',
	u'YT': u'Mayotte',
	u'UM': u'United States Minor Outlying Islands',
	u'LB': u'Lebanon',
	u'LC': u'Saint Lucia',
	u'LA': u'Lao People\'s Democratic Republic',
	u'TV': u'Tuvalu',
	u'TW': u'Taiwan',
	u'TT': u'Trinidad and Tobago',
	u'TR': u'Turkey',
	u'LK': u'Sri Lanka',
	u'LI': u'Liechtenstein',
	u'LV': u'Latvia',
	u'TO': u'Tonga',
	u'LT': u'Lithuania',
	u'LU': u'Luxembourg',
	u'LR': u'Liberia',
	u'LS': u'Lesotho',
	u'TH': u'Thailand',
	u'TF': u'French Southern Territories',
	u'TG': u'Togo',
	u'TD': u'Chad',
	u'TC': u'Turks and Caicos Islands',
	u'LY': u'Libyan Arab Jamahiriya',
	u'VA': u'Vatican City State (Holy See)',
	u'VC': u'Saint Vincent and The Grenadines',
	u'AE': u'United Arab Emirates',
	u'AD': u'Andorra',
	u'AG': u'Antigua and Barbuda',
	u'AF': u'Afghanistan',
	u'AI': u'Anguilla',
	u'VI': u'Virgin Islands (U.S.)',
	u'IS': u'Iceland',
	u'IR': u'Iran (Islamic Republic of)',
	u'AM': u'Armenia',
	u'AL': u'Albania',
	u'AO': u'Angola',
	u'AN': u'Netherlands Antilles',
	u'AQ': u'Antarctica',
	u'AS': u'American Samoa',
	u'AR': u'Argentina',
	u'AU': u'Australia',
	u'AT': u'Austria',
	u'AW': u'Aruba',
	u'IN': u'India',
	u'TZ': u'Tanzania, United Republic of',
	u'AZ': u'Azerbaijan',
	u'IE': u'Ireland',
	u'ID': u'Indonesia',
	u'UA': u'Ukraine',
	u'QA': u'Qatar',
	u'MZ': u'Mozambique',
	u'BA': u'Bosnia and Herzegovina',
	u'CD': u'Congo, The Democratic Republic of the',
	u'CS': u'Serbia and Montenegro',
	u'HR': u'Croatia',
	u'KP': u'Korea (North), Democratic People\'s Republic of',
	u'KR': u'Korea (South), Republic of',
	u'SK': u'Slovakia',
	u'SU': u'Soviet Union (historical, 1922-1991)',
	u'TL': u'East Timor',
	u'XC': u'Czechoslovakia (historical, 1918-1992)',
	u'XE': u'Europe',
	u'XG': u'East Germany (historical, 1949-1990)',
	u'XU': u'[Unknown Country]',
	u'XW': u'[Worldwide]',
	u'YU': u'Yugoslavia (historical, 1918-1992)',
}

# EOF
